#!/usr/bin/python

from tf.transformations import euler_from_quaternion
import numpy as np
import rosbag
import argparse
import sys
import pdb
import csv

class ConversionUtils:
    @staticmethod
    def quat2Yaw(quat):
        orientation_list = [quat.x, quat.y, quat.z, quat.w]
        (roll, pitch, yaw) = euler_from_quaternion (orientation_list)
        return yaw
    @staticmethod
    def clampAngle(angle):
        while (angle >= np.pi):
            angle -= np.pi
        while (angle < -np.pi):
            angle += np.pi
        return angle

class WheelRotationWriter:
    def __init__(self):
        self.leftCnt = []
        self.rightCnt = []
        self.leftSpd = []
        self.rightSpd = []
        self.t = []
        self.t_stamp = []
        self.topic = ["/sensors/odometry/wheel_rotation"]
    def get_topics(self):
        return self.topic
    def parse(self, topic, msg, t):
        if (topic == self.topic[0]):
            scale = 1#4096 * 10
            self.t.append(t)
            self.t_stamp.append(msg.header.stamp.secs + msg.header.stamp.nsecs * 1e-9)
            self.leftCnt.append(msg.leftWheelAbsRotationCount * scale)
            self.rightCnt.append(msg.rightWheelAbsRotationCount * scale)
            if (len(self.leftCnt) > 1):
                leftSpd = (self.leftCnt[-1] - self.leftCnt[-2]) #/ (self.t_stamp[-1] - self.t_stamp[-2])
                self.leftSpd.append(leftSpd)
            else:
                self.leftSpd.append(0)
            if (len(self.rightCnt) > 1):
                rightSpd = (self.rightCnt[-1] - self.rightCnt[-2]) #/ (self.t_stamp[-1] - self.t_stamp[-2])
                #if rightSpd > 0.001:
                #    print "-1: {}, -2: {}, spd: {}".format(self.rightCnt[-1], self.rightCnt[-2], rightSpd)
                self.rightSpd.append(rightSpd)
            else:
                self.rightSpd.append(0)           
    def writeCSV(self, prefix):
        print "Writing WheelRotation to file"
        with open('{}_wheel_rotation.csv'.format(prefix), 'w+') as f:
            f.write("%time, left_wheel_rotation, right_wheel_rotation\n")
            for k in range(len(self.t_stamp)):
                f.write("{}, {}, {}\n".format(self.t_stamp[k], self.leftCnt[k], self.rightCnt[k]))

class AmclPoseWriter:
    def __init__(self):
        self.amcl_x = []
        self.amcl_y = []
        self.amcl_theta = []
        self.t = []
        self.t_stamp = []
        self.topic = ["/amcl_pose"]
    def get_topics(self):
        return self.topic
    def parse(self, topic, msg, t):
        if (topic == self.topic[0]):
            self.t.append(t)
            self.t_stamp.append(msg.header.stamp.secs + msg.header.stamp.nsecs * 1e-9)
            self.amcl_x.append(msg.pose.pose.position.x)
            self.amcl_y.append(msg.pose.pose.position.y)
            self.amcl_theta.append(ConversionUtils.quat2Yaw(msg.pose.pose.orientation))
    def writeCSV(self, prefix):
        print "Writing AMCLPose to file"
        with open('{}_amcl_pose.csv'.format(prefix), 'w+') as f:
            f.write("%time, x, y, theta\n")
            for k in range(len(self.t_stamp)):
                f.write("{}, {}, {}, {}\n".format(self.t_stamp[k], self.amcl_x[k], self.amcl_y[k], self.amcl_theta[k]))

class IMUWriter:
    def __init__(self):
        self.t = []
        self.t_stamp = []
        self.orientation_x = []
        self.orientation_y = []
        self.orientation_z = []
        self.orientation_w = []
        self.orientation_covariance = []
        self.angular_velocity_x = []
        self.angular_velocity_y = []
        self.angular_velocity_z = []
        self.angular_velocity_covariance = []
        self.linear_acceleration_x = []
        self.linear_acceleration_y = []
        self.linear_acceleration_z = []
        self.linear_acceleration_covariance = []
        self.topic = ["/sensors/odometry/imu"]
    def get_topics(self):
        return self.topic
    def parse(self, topic, msg, t):
        if (topic == self.topic[0]):
            self.t.append(t)
            self.t_stamp.append(msg.header.stamp.secs + msg.header.stamp.nsecs * 1e-9)
            self.orientation_x.append(msg.orientation.x)
            self.orientation_y.append(msg.orientation.y)
            self.orientation_z.append(msg.orientation.z)
            self.orientation_w.append(msg.orientation.w)
            self.orientation_covariance.append(msg.orientation_covariance)
            self.angular_velocity_x.append(msg.angular_velocity.x)
            self.angular_velocity_y.append(msg.angular_velocity.y)
            self.angular_velocity_z.append(msg.angular_velocity.z)
            self.angular_velocity_covariance.append(msg.angular_velocity_covariance)
            self.linear_acceleration_x.append(msg.linear_acceleration.x)
            self.linear_acceleration_y.append(msg.linear_acceleration.y)
            self.linear_acceleration_z.append(msg.linear_acceleration.z)
            self.linear_acceleration_covariance.append(msg.linear_acceleration_covariance)
    def writeCSV(self, prefix):
        print "Writing IMU data to file"
        with open('{}_imu_data.csv'.format(prefix), 'w+') as f:
            f.write("%time, orientation_x, orientation_y, orientation_z, orientation_w, orientation_covariance_0, orientation_covariance_1,orientation_covariance_2, orientation_covariance_3, orientation_covariance_4, orientation_covariance_5, orientation_covariance_6, orientation_covariance_7, orientation_covariance_8, angular_velocity_x, angular_velocity_y, angular_velocity_z, angular_velocity_covariance_0, angular_velocity_covariance_1, angular_velocity_covariance_2, angular_velocity_covariance_3, angular_velocity_covariance_4, angular_velocity_covariance_5,angular_velocity_covariance_6, angular_velocity_covariance_7, angular_velocity_covariance_8, linear_acceleration_x, linear_acceleration_y, linear_acceleration_z, linear_acceleration_covariance_0, linear_acceleration_covariance_1, linear_acceleration_covariance_2, linear_acceleration_covariance_3, linear_acceleration_covariance_4, linear_acceleration_covariance_5,linear_acceleration_covariance_6, linear_acceleration_covariance_7, linear_acceleration_covariance_8\n")
            for k in range(len(self.t_stamp)):
                f.write("{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}\n".format(self.t_stamp[k], self.orientation_x[k], self.orientation_y[k], self.orientation_z[k], self.orientation_w[k], self.orientation_covariance[k], self.angular_velocity_x[k], self.angular_velocity_y[k], self.angular_velocity_z[k], self.angular_velocity_covariance[k], self.linear_acceleration_x[k], self.linear_acceleration_y[k], self.linear_acceleration_z[k], self.linear_acceleration_covariance[k]))

class VelocityCommandPlotter:
    def __init__(self):
        self.vx = []
        self.vy = []
        self.vz = []
        self.omega_x = []
        self.omega_y = []
        self.omega_z = []
        self.t = []
        self.topic = ["/sensors/odometry/velocity/cmd"]
    def get_topics(self):
        return self.topic
    def parse(self, topic, msg, t):
        if (topic == self.topic[0]):
            self.t.append(t.to_sec())
            self.vx.append(msg.linear.x)
            self.vy.append(msg.linear.y)
            self.vz.append(msg.linear.z)
            self.omega_x.append(msg.angular.x)
            self.omega_y.append(msg.angular.y)
            self.omega_z.append(msg.angular.z)
           
    def writeCSV(self, prefix):
        print "Writing Velocity Command to file"
        with open('{}_velocity_cmd.csv'.format(prefix), 'w+') as f:
            f.write("%time, linear_velocity_x, linear_velocity_y, linear_velocity_z, angular_velocity_x, angular_velocity_y, angular_velocity_z\n")
            for k in range(len(self.t)):
                f.write("{}, {}, {}, {}, {}, {}, {}\n".format(self.t[k], self.vx[k], self.vy[k], self.vz[k], self.omega_x[k], self.omega_y[k], self.omega_z[k]))

class PosePlotter:
    def __init__(self):
        self.pose_x = []
        self.pose_y = []
        self.pose_z = []
        self.orientation_x = []
        self.orientation_y = []
        self.orientation_z = []
        self.orientation_w = []
        self.twist_x = []
        self.twist_y = []
        self.twist_z = []
        self.twist_angular_x = []
        self.twist_angular_y = []
        self.twist_angular_z = []
        self.t = []
        self.t_stamp = []
        self.topic = ["/sensors/odometry/pose"]
    def get_topics(self):
        return self.topic
    def parse(self, topic, msg, t):
        if (topic == self.topic[0]):
            self.t.append(t.to_sec())
            self.t_stamp.append(msg.header.stamp.secs + msg.header.stamp.nsecs * 1e-9)
            self.pose_x.append(msg.pose.pose.position.x)
            self.pose_y.append(msg.pose.pose.position.y)
            self.pose_z.append(msg.pose.pose.position.z)
            self.orientation_x.append(msg.pose.pose.orientation.x)
            self.orientation_y.append(msg.pose.pose.orientation.y)
            self.orientation_z.append(msg.pose.pose.orientation.z)
            self.orientation_w.append(msg.pose.pose.orientation.w)
            self.twist_x.append(msg.twist.twist.linear.x)
            self.twist_y.append(msg.twist.twist.linear.y)
            self.twist_z.append(msg.twist.twist.linear.z)
            self.twist_angular_x.append(msg.twist.twist.angular.x)
            self.twist_angular_y.append(msg.twist.twist.angular.y)
            self.twist_angular_z.append(msg.twist.twist.angular.z)
    def writeCSV(self, prefix):
        print "Writing Pose Data to file"
        with open('{}_pose.csv'.format(prefix), 'w+') as f:
            f.write("%time, pose_x, pose_y, pose_z, orientation_x, orientation_y, orientation_z, orientation_w, twist_linear_x, twist_linear_y, twist_linear)z, twist_angular_x, twist_angular_y, twist_angular_z\n")
            for k in range(len(self.t_stamp)):
                f.write("{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}\n".format(self.t_stamp[k], self.pose_x[k], self.pose_y[k], self.pose_z[k], self.orientation_x[k], self.orientation_y[k], self.orientation_z[k], self.orientation_w[k], self.twist_x[k], self.twist_y[k], self.twist_z[k], self.twist_angular_x[k], self.twist_angular_y[k], self.twist_angular_z[k]))



class BagCsvWriter:
    def __init__(self):
        # Create some csv writers
        self.writers = []
        self.writers.append(WheelRotationWriter())
        self.writers.append(AmclPoseWriter())
        self.writers.append(IMUWriter())
        self.writers.append(VelocityCommandPlotter())
        self.writers.append(PosePlotter())

    def run(self, bag_name):
        print "Parsing data from bag " + bag_name
        # Get the list of topics
        topics_to_write = []
        for writer in self.writers:
            topics_to_write.extend(writer.get_topics())
        # Open the bag and parse all of the messages
        with rosbag.Bag(bag_name, 'r') as bag:
            for topic, msg, t in bag.read_messages(topics=topics_to_write):
                for writer in self.writers:
                    writer.parse(topic, msg, t)

        # Let the bag close and write the data now
        print "Writing csv files"
        for writer in self.writers:
            try:
                writer.writeCSV(bag_name.rstrip('.bag'))
            except AttributeError:
                print "Caught error"


        print "Done"

if __name__ == '__main__':
    bag_name = "test.bag"
    if len(sys.argv) < 2:
        print "No bag name supplied.  Trying " + bag_name
    else:
        bag_name = sys.argv[1]
    bp = BagCsvWriter()
    bp.run(bag_name)
    print "Done plotting"


#!/usr/bin/env python
# Print total cumulative serialized msg size per topic
#how to use = python bagSize.py <bag_name> <topic to look > <topic2 to look at> <topic3 to look at>
import rosbag
import sys
from texttable import Texttable



def sizeof_fmt(num):
    for letter in ['','K','M','G']:
        if abs(num) < 1024.0:
            return "%3.2f%sB" % (num, letter)
        num /= 1024.0


observed_topics = []
numArgs = len(sys.argv)
for i in range(2, (numArgs)):
  observed_topics.append(sys.argv[i])

topic_size_dict = {}
for topic, msg, time in rosbag.Bag(sys.argv[1], 'r').read_messages(raw=True):
  topic_size_dict[topic] = topic_size_dict.get(topic, 0) + len(msg[1])
topic_size_list = list(topic_size_dict.items())
topic_size_list.sort(key=lambda x: x[1])
totalSize = 0
for topic, size in topic_size_list:
  totalSize = totalSize + size

t = Texttable()
t.add_row(['Topic Name', 'Size', 'Percentage of bag'])
for topic, size in topic_size_list:
  t.add_row([topic, sizeof_fmt(size), "{0:.1%}".format(float(size)/float(totalSize))])
print(t.draw())

print ""
print "Total size of the bag %s" % sizeof_fmt(totalSize)
print ""

t2 = Texttable()
t2.add_row(['Topic Name', 'Size', 'Percentage of bag'])
print observed_topics
for topic, size in topic_size_list:
  for top in observed_topics:
    if top == topic:
      t2.add_row([topic, sizeof_fmt(size), "{0:.1%}".format(float(size)/float(totalSize))])
print(t2.draw())


import rosbag
import sys
import matplotlib.pyplot as plt
import tf


if (len(sys.argv) != 2):
	print "invalid number of arguments:   " + str(len(sys.argv))
	sys.exit(1)
else:
	bagFile = sys.argv[1]
	print "reading: " + str(bagFile)

oldbag = rosbag.Bag(bagFile)

xs = []
ys = []
thetas = []
ts = []


for topic, msg, t in oldbag.read_messages():
	if topic == "/tf":
		shouldWrite = True
		for i in range(len(msg.transforms)):
			if msg.transforms[i].child_frame_id == "odom":
				quaternion = (
					0,
					0,
					msg.transforms[i].transform.rotation.z,
					msg.transforms[i].transform.rotation.w)
				xs.append(msg.transforms[i].transform.translation.x)
				ys.append(msg.transforms[i].transform.translation.y)
				thetas.append(tf.transformations.euler_from_quaternion(quaternion)[2])
				ts.append(t.to_sec())
plt.figure(0)
plt.plot(ts, thetas, 'k-*')
plt.xlabel('Time (s)')
plt.ylabel('Displacement (rad)')
plt.title('AMCL correction of theta over time while spinning')
plt.show()




import rosbag
import sys
#verify correct input arguments: 1 or 2

def print_progress(iteration, total, prefix='', suffix='', decimals=1, bar_length=100):
	"""
	Call in a loop to create terminal progress bar
	@params:
		iteration   - Required  : current iteration (Int)
		total       - Required  : total iterations (Int)
		prefix      - Optional  : prefix string (Str)
		suffix      - Optional  : suffix string (Str)
		decimals    - Optional  : positive number of decimals in percent complete (Int)
		bar_length  - Optional  : character length of bar (Int)
	"""
	str_format = "{0:." + str(decimals) + "f}"
	percents = str_format.format(100 * (iteration / float(total)))
	filled_length = int(round(bar_length * iteration / float(total)))
	bar = ' ' * filled_length + '-' * (bar_length - filled_length)

	sys.stdout.write('\r%s |%s| %s%s %s' % (prefix, bar, percents, '%', suffix)),

	if iteration == total:
		sys.stdout.write('\n')
	sys.stdout.flush()




if (len(sys.argv) != 2):
	print "invalid number of arguments:   " + str(len(sys.argv))
	print "should be 2: 'bag2csv.py' and 'bagName'"
	sys.exit(1)
else:
	bagFile = sys.argv[1]
	print "reading: " + str(bagFile)

oldbag = rosbag.Bag(bagFile)
bagName = oldbag.filename
newBagName = bagName + "-noAccelnoMap.bag"

total = oldbag.get_message_count()

prevImuTime = None
print_progress(0, total, prefix = 'Progress:', suffix = 'Complete', bar_length = 50)
with rosbag.Bag(newBagName, 'w') as outbag:
	num = 0
	for topic, msg, t in oldbag.read_messages():
		num = num + 1
		if topic == "/sensors/odometry/imu":
			msg2 = msg
			msg2.linear_acceleration.x = 0;
			msg2.linear_acceleration.y = 0;
			msg2.linear_acceleration.z = 1;
			outbag.write(topic, msg2, t)

		if topic == "/tf":
			shouldWrite = True
			for i in range(len(msg.transforms)):
				if msg.transforms[i].child_frame_id == "odom":
					shouldWrite = False
			if(shouldWrite):
				outbag.write(topic, msg, t)
		else:
			outbag.write(topic, msg, t)
		if(num%1000 == 0):
			print_progress(num, total, prefix = 'Progress:', suffix = 'Complete', bar_length = 50)



# Have to use custom impl of Hough Transform (separate from openCV) because
# of no access to the accumulator matrix
import cv2
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.image as mpimg
import matplotlib.pylab as pl
import matplotlib.gridspec as gridspec
import sys

def setup(filename): 
    orig_img = cv2.imread(filename,)
    # get edges
    edges = cv2.Canny(orig_img, threshold1 = 0, threshold2 = 50, apertureSize = 3)

    return edges

def test2():
    print "hello"
# http://nabinsharma.wordpress.com/2012/12/26/linear-hough-transform-using-python/
def hough_transform(img_bin, theta_res=1.0, rho_res=1.0):
    nR, nC = img_bin.shape
    theta = np.linspace(-90, 0, int(np.ceil(90.0/theta_res)) + 1)
    theta = np.concatenate((theta, -theta[len(theta)-2::-1]))

    D = np.sqrt((nR - 1)**2 + (nC -1)**2)
    q = int(np.ceil(D/rho_res))
    nrho = 2*q + 1
    rho = np.linspace(-q*rho_res, q*rho_res, nrho)
    H = np.zeros((len(rho), len(theta)))

    for rowIdx in range(nR):
        for colIdx in range(nC):
            if img_bin[rowIdx, colIdx]:
                for thIdx in range(len(theta)):
                  rhoVal = colIdx * np.cos(theta[thIdx]*np.pi/180) + rowIdx*np.sin(theta[thIdx]*np.pi/180)
                  rhoIdx = np.nonzero(np.abs(rho-rhoVal) == np.min(np.abs(rho-rhoVal)))[0]
                  H[rhoIdx[0], thIdx] += 1

    return rho, theta, H


def gen_data(file_in, houghout):
    edges = setup(file_in)
    rho, theta, H = hough_transform(edges, theta_res=1.0, rho_res=1.0)
   
    # save accumulator matrix
    np.save(houghout, H)

if __name__ == '__main__':

    gen_data(sys.argv[1], 'hough.txt')



#!/usr/bin/env python
'''
This script saves each topic in a bagfile as a csv.

Accepts a filename as an optional argument. Operates on all bagfiles in current directory if no argument provided

Usage1 (for one bag file):
	python bag2csv.py filename.bag
Usage 2 (for all bag files in current directory):
	python bag2csv.py

Written by Nick Speal in May 2013 at McGill University's Aerospace Mechatronics Laboratory
www.speal.ca

Supervised by Professor Inna Sharf, Professor Meyer Nahon 

'''

import rosbag, sys, csv
import time
import string
import os #for file management make directory
import shutil #for file management, copy file

#verify correct input arguments: 1 or 2
if (len(sys.argv) > 2):
	print "invalid number of arguments:   " + str(len(sys.argv))
	print "should be 2: 'bag2csv.py' and 'bagName'"
	print "or just 1  : 'bag2csv.py'"
	sys.exit(1)
elif (len(sys.argv) == 2):
	listOfBagFiles = [sys.argv[1]]
	print "reading only 1 bagfile: " + str(listOfBagFiles[0])
	numberOfFiles = str(1)
elif (len(sys.argv) == 1):
	listOfBagFiles = [f for f in os.listdir(".") if f[-4:] == ".bag"]	#get list of only bag files in current dir.
	numberOfFiles = str(len(listOfBagFiles))
	print "reading all " + numberOfFiles + " bagfiles in current directory: \n"
	for f in listOfBagFiles:
		print f
	print "\n press ctrl+c in the next 10 seconds to cancel \n"
	time.sleep(10)
else:
	print "bad argument(s): " + str(sys.argv)	#shouldnt really come up
	sys.exit(1)

count = 0
for bagFile in listOfBagFiles:
	count += 1
	print "reading file " + str(count) + " of  " + numberOfFiles + ": " + bagFile
	#access bag
	bag = rosbag.Bag(bagFile)
	bagContents = bag.read_messages()
	bagName = bag.filename


	#create a new directory
	folder = string.rstrip(bagName, ".bag")
	try:	#else already exists
		os.makedirs(folder)
	except:
		pass
	shutil.copyfile(bagName, folder + '/' + bagName)


	#get list of topics from the bag
	listOfTopics = []
	for topic, msg, t in bagContents:
		if topic not in listOfTopics:
			listOfTopics.append(topic)


	for topicName in listOfTopics:
		#Create a new CSV file for each topic
		filename = folder + '/' + string.replace(topicName, '/', '_slash_') + '.csv'
		with open(filename, 'w+') as csvfile:
			filewriter = csv.writer(csvfile, delimiter = ',')
			firstIteration = True	#allows header row
			for subtopic, msg, t in bag.read_messages(topicName):	# for each instant in time that has data for topicName
				#parse data from this instant, which is of the form of multiple lines of "Name: value\n"
				#	- put it in the form of a list of 2-element lists
				msgString = str(msg)
				msgList = string.split(msgString, '\n')
				instantaneousListOfData = []
				for nameValuePair in msgList:
					splitPair = string.split(nameValuePair, ':')
					for i in range(len(splitPair)):	#should be 0 to 1
						splitPair[i] = string.strip(splitPair[i])
					instantaneousListOfData.append(splitPair)
				#write the first row from the first element of each pair
				if firstIteration:	# header
					headers = ["rosbagTimestamp"]	#first column header
					for pair in instantaneousListOfData:
						headers.append(pair[0])
					filewriter.writerow(headers)
					firstIteration = False
				# write the value from each pair to the file
				values = [str(t)]	#first column will have rosbag timestamp
				for pair in instantaneousListOfData:
					values.append(pair[1])
				filewriter.writerow(values)
	bag.close()
print "Done reading all " + numberOfFiles + " bag files."



#!/usr/bin/env python
# The code is from here (https://gist.github.com/NikolausDemmel/8944211)

import sys
import argparse
from fnmatch import fnmatchcase

from rosbag import Bag

def main():

    parser = argparse.ArgumentParser(description='Merge one or more bag files with the possibilities of filtering topics.')
    parser.add_argument('outputbag',
                        help='output bag file with topics merged')
    parser.add_argument('inputbag', nargs='+',
                        help='input bag files')
    parser.add_argument('-v', '--verbose', action="store_true", default=False,
                        help='verbose output')
    parser.add_argument('-t', '--topics', default="*",
                        help='string interpreted as a list of topics (wildcards \'*\' and \'?\' allowed) to include in the merged bag file')

    args = parser.parse_args()

    topics = args.topics.split(' ')

    total_included_count = 0
    total_skipped_count = 0

    if (args.verbose):
        print("Writing bag file: " + args.outputbag)
        print("Matching topics against patters: '%s'" % ' '.join(topics))

    with Bag(args.outputbag, 'w') as o: 
        for ifile in args.inputbag:
            matchedtopics = []
            included_count = 0
            skipped_count = 0
            if (args.verbose):
                print("> Reading bag file: " + ifile)
            with Bag(ifile, 'r') as ib:
                for topic, msg, t in ib:
                    if any(fnmatchcase(topic, pattern) for pattern in topics):
                        if not topic in matchedtopics:
                            matchedtopics.append(topic)
                            if (args.verbose):
                                print("Including matched topic '%s'" % topic)
                        o.write(topic, msg, t)
                        included_count += 1
                    else:
                        skipped_count += 1
            total_included_count += included_count
            total_skipped_count += skipped_count
            if (args.verbose):
                print("< Included %d messages and skipped %d" % (included_count, skipped_count))

    if (args.verbose):
        print("Total: Included %d messages and skipped %d" % (total_included_count, total_skipped_count))

if __name__ == "__main__":
    main()



#!/usr/bin/python
import math



def wrapAngle(angle):
    while angle >  math.pi:
        angle -= 2 * math.pi
    while angle <= -math.pi:
        angle += 2 * math.pi
    return angle



class State:
    def __init__(self, x, y, theta):
        self.x = x
        self.y = y
        self.theta = theta

    def __str__(self):
        return "[pos: [{}, {}, {}]]".format(
            self.x, self.y, self.theta)

    def __repr__(self):
    	return str(self)

    def transform(self, p):
        return State(
            self.x + p.x * math.cos(self.theta) - p.y * math.sin(self.theta),
            self.y + p.x * math.sin(self.theta) + p.y * math.cos(self.theta),
            wrapAngle(self.theta + p.theta))
    def rotate(self, angle):
    	return State(self.x * math.cos(angle) - self.y * math.sin(angle), 
    		self.x * math.sin(angle) + self.y * math.cos(angle),
    		self.theta + angle)
    def translate(self, x, y):
    	return State(self.x + x, 
    		self.y + y,
    		self.theta)

    def inverse(self):
        return State(
            -self.x * math.cos(self.theta) - self.y * math.sin(self.theta),
            -self.y * math.cos(self.theta) + self.x * math.sin(self.theta),
            wrapAngle(-self.theta))


import setuptools

with open("README.md", "r") as fh:
	long_description = fh.read()

setuptools.setup(
	name="sixriver-ros-utils",
	version="0.0.3",
	author="sixriver",
	author_email="dniktopoulos@6river.com",
	description="Ros util for python",
	long_description=long_description,
	long_description_content_type="text/markdown",
	url="https://github.com/6RiverSystems/mfp_tools",
	packages=setuptools.find_packages(),
	python_requires='>=2.7',

)